const mongoose = require('../../common/services/mongoose.service').mongoose;
const Schema = mongoose.Schema;
const {queryFormatter,queryBuilder_string,
    queryBuilder_number,
    queryBuilder_date,
    queryBuilder_array,
    queryBuilder_range_array} = require("../../common/functions/queryutilMongo")
const headersSchema = new Schema({
    	createBy : { type: String,required:true,default:''},
			createAt : { type: Date,required:true},
			updateBy : { type: String},
			updateAt : { type: Date},
			headerItems : { type: String,required:true,default:'',maxLength:50,minLength:2}
});

headersSchema.virtual('id').get(function () {
    return this._id.toHexString();
});

// Ensure virtual fields are serialised.
headersSchema.set('toJSON', {
    virtuals: true
});

headersSchema.findById = function (cb) {
    return this.model('Headers').find({id: this.id}, cb);
};

const Headers = mongoose.model('Headers', headersSchema);


exports.findById = (id,extraField) => {
    var extraQuery =queryFormatter(extraField);
    var queries = {...extraQuery,_id:id}
    return Headers.findOne(queries)
        .then((result) => {
            result = result.toJSON();
            delete result._id;
            delete result.__v;
            return result;
        });
};

exports.createHeaders = (headersData) => {
    return new Promise((resolve, reject) => {
    
    const headers = new Headers(headersData);
    headers.save(function (err, saved) {
        if (err) {
            return reject(err);
        }
        resolve(saved)
    });
    });
};

exports.list = (perPage, page , query ) => {
        var _query={};
        let sortBy='_id'
        let sortDirection=-1
        
    if(query.createBy){
        let createBy_= queryBuilder_string(query,'createBy');
        _query={..._query,...createBy_}
    }


        if(query.createAt){
            let createAt_= queryBuilder_date(query,'createAt');
            _query={..._query,...createAt_}
        }
        if(query.createAt_array){
            let createAt_a= queryBuilder_range_array(query,'createAt',"date");
            _query={..._query,...createAt_a}
        }
    
    

    if(query.updateBy){
        let updateBy_= queryBuilder_string(query,'updateBy');
        _query={..._query,...updateBy_}
    }


        if(query.updateAt){
            let updateAt_= queryBuilder_date(query,'updateAt');
            _query={..._query,...updateAt_}
        }
        if(query.updateAt_array){
            let updateAt_a= queryBuilder_range_array(query,'updateAt',"date");
            _query={..._query,...updateAt_a}
        }
    
    

    if(query.headerItems){
        let headerItems_= queryBuilder_string(query,'headerItems');
        _query={..._query,...headerItems_}
    }

        if(query.sortBy){
            sortBy = query.sortBy;
        }
        if(query.sortDirection){
            sortDirection = query.sortDirection;
        }
        var sortBoj={[sortBy]:sortDirection};
        return new Promise((resolve, reject) => {
        Headers.find(_query)
            .limit(perPage)
            .sort(sortBoj)
            .skip(perPage * page)
            .exec(function (err, headers) {
                if (err) {
                    reject(err);
                } else {
                    Headers.countDocuments(_query).exec().then((total)=>{
                        const promises = { docs: headers , count: total ,perpage:perPage,page:page };
                        resolve(promises);
                    }).catch((err2)=>{
                        reject(err2);
                    })
                }
            })
    });
};
exports.listAll = ( query={} ) => {
    var _query={...query};
    let sortBy='_id'
    let sortDirection=-1
    let max_limit = 300;
    if(query.sortBy){
        sortBy = query.sortBy;
    }
    if(query.sortDirection){
        sortDirection = query.sortDirection;
    }
    var sortBoj={[sortBy]:sortDirection};
    return new Promise((resolve, reject) => {
    Headers.find(_query)
        .limit(max_limit) 
        .sort(sortBoj)
        .exec(function (err, headers) {
                if (err) {
                    reject(err);
                } else {
                resolve(headers);
                }
            })
    });
};
exports.listSuggestions = (query ) => {
    var { search, ...queryWithoutSearch } = query
    var _query=queryWithoutSearch
    if(search){
        _query['$or'] =[ 
            {"headerItems": {$regex: search, $options: "i"}}
        ]
    }
    return new Promise((resolve, reject) => {
        Headers.find(_query)
            .exec(function (err, headers) {
                if (err) {
                    reject(err);
                } else {
                    resolve(headers);
                }
            })
    });
};
exports.patchHeaders = (id, headersData,extraField={}) => {
    
    var extraQuery =queryFormatter(extraField);
    var queries = {...extraQuery,_id:id}
    return new Promise((resolve, reject) => {
        
        Headers.findOne(queries, function (err, headers) {
            if (err) reject(err);
            for (let i in headersData) {
                headers[i] = headersData[i];
            }
            headers.save(function (err, updatedHeaders) {
                if (err) return reject(err);
                resolve(updatedHeaders);
            });
        });
    })

};

exports.removeById = (headersId,extraField={}) => {
    var extraQuery =queryFormatter(extraField);
    var queries = {...extraQuery,_id:headersId}
    return new Promise((resolve, reject) => {
        Headers.findOneAndDelete(queries, (err) => {
            if (err) {
                reject(err);
            } else {
                resolve(err);
            }
        });
    });
};



    