const mongoose = require('../../common/services/mongoose.service').mongoose;
const Schema = mongoose.Schema;
const {queryFormatter,queryBuilder_string,
    queryBuilder_number,
    queryBuilder_date,
    queryBuilder_array,
    queryBuilder_range_array} = require("../../common/functions/queryutilMongo")
const itemsSchema = new Schema({
    	createBy : { type: String,required:true,default:''},
			createAt : { type: Date,required:true},
			updateBy : { type: String},
			updateAt : { type: Date},
			itemName : { type: String,required:true,default:'',maxLength:50,minLength:2},
            parentObject : {type: String,default:null},
            isArray : {type: Boolean,default:false},
            arrayOf : {type: String,default:null},
			required : { type:Boolean,default:false},
			validation : { type: String},
			itemType : { type: String,required:true,default:'Body'},
			fileTypes : { type:[String]},
			endPoint : { type: Schema.Types.ObjectId,required:false,ref:'Endpoints'},
			project : { type: Schema.Types.ObjectId,required:true,ref:'Project'},
			value : { type: String},
            valueLength : { type: Number,default:0},
            maxValueLength : { type: Number,default:0},
            submitAs : { type: String,default:null},
            itemPath: { type: String,default:null},
            groupCheckItem: { type: String,default:null},
            groupCheckValue: { type: String,default:null},
            

            
});

itemsSchema.virtual('id').get(function () {
    return this._id.toHexString();
});

// Ensure virtual fields are serialised.
itemsSchema.set('toJSON', {
    virtuals: true
});

itemsSchema.findById = function (cb) {
    return this.model('Items').find({id: this.id}, cb);
};

const Items = mongoose.model('Items', itemsSchema);


exports.findById = (id,extraField) => {
    var extraQuery =queryFormatter(extraField);
    var queries = {...extraQuery,_id:id}
    return Items.findOne(queries)
    .populate({ path: 'project', select: 'id projectName' })
    .populate({ path: 'endPoint', select: 'id name path' })
        .then((result) => {
            result = result.toJSON();
            delete result._id;
            delete result.__v;
            return result;
        });
};

exports.createItems = (itemsData) => {
    return new Promise(async(resolve, reject) => {

        let itemExist= await this.listAll({endPoint:itemsData.endPoint,itemName:itemsData.itemName,itemType:itemsData.itemType})
      
        console.log(itemExist)
       if (itemExist && itemExist.length>0) {
        return reject("Item Name  already exists");
        
    }
    //console.log(itemsData)
    const items = new Items(itemsData);
    items.save(function (err, saved) {
        if (err) {
            return reject(err);
        }
        resolve(saved)
    });
    });
};

exports.list = (perPage, page , query ) => {
        var _query={};
        let sortBy='_id'
        let sortDirection=-1
        
    if(query.createBy){
        let createBy_= queryBuilder_string(query,'createBy');
        _query={..._query,...createBy_}
    }


        if(query.createAt){
            let createAt_= queryBuilder_date(query,'createAt');
            _query={..._query,...createAt_}
        }
        if(query.createAt_array){
            let createAt_a= queryBuilder_range_array(query,'createAt',"date");
            _query={..._query,...createAt_a}
        }
    
    

    if(query.updateBy){
        let updateBy_= queryBuilder_string(query,'updateBy');
        _query={..._query,...updateBy_}
    }


        if(query.updateAt){
            let updateAt_= queryBuilder_date(query,'updateAt');
            _query={..._query,...updateAt_}
        }
        if(query.updateAt_array){
            let updateAt_a= queryBuilder_range_array(query,'updateAt',"date");
            _query={..._query,...updateAt_a}
        }
    
    

    if(query.itemName){
        let itemName_= queryBuilder_string(query,'itemName');
        _query={..._query,...itemName_}
    }


    if(query.required!=null){
        _query['required'] = query.required 
    }
            

    if(query.itemType){
        let itemType_= queryBuilder_array(query,'itemType');
        _query={..._query,...itemType_}
    }
    
    if(query.endPoint){
        if(query.endPoint=="empty"){
            endPoint_={endPoint:null}
            _query={..._query,...endPoint_}
        }else{
        let endPoint_= queryBuilder_string(query,'endPoint');
        _query={..._query,...endPoint_}
        }
    }


    if(query.fileTypes){
        if(Array.isArray(query.fileTypes)){
            const regexfileTypes = query.fileTypes.map(value => new RegExp(value, 'i'));
            _query['fileTypes'] = { $in: regexfileTypes } 
        }else{
            _query['fileTypes'] = { $in: [new RegExp(query.fileTypes, "i")] } 
        }
    }
        

    if(query.project){
        let project_= queryBuilder_string(query,'project');
        _query={..._query,...project_}
    }


    if(query.value){
        let value_= queryBuilder_string(query,'value');
        _query={..._query,...value_}
    }

        if(query.sortBy){
            sortBy = query.sortBy;
        }
        if(query.sortDirection){
            sortDirection = parseInt(query.sortDirection);
        }
        var sortBoj={[sortBy]:sortDirection};
        console.log(sortBoj)
        return new Promise((resolve, reject) => {
        Items.find(_query)
            .populate({ path: 'project', select: 'id projectName' })
            .populate({ path: 'endPoint', select: 'id name path' })
            .limit(perPage)
            .sort(sortBoj)
            .skip(perPage * page)
            .exec(function (err, items) {
                if (err) {
                    reject(err);
                } else {
                    Items.countDocuments(_query).exec().then((total)=>{
                        const promises = { docs: items , count: total ,perpage:perPage,page:page };
                        resolve(promises);
                    }).catch((err2)=>{
                        reject(err2);
                    })
                }
            })
    });
};
exports.listAll = ( query={} ) => {
    var _query={...query};
    let sortBy='_id'
    let sortDirection=-1
    let max_limit = 30000;
    if(query.sortBy){
        sortBy = query.sortBy;
    }
    if(query.sortDirection){
        sortDirection = query.sortDirection;
    }
    var sortBoj={[sortBy]:sortDirection};
    console.log(_query)
    return new Promise((resolve, reject) => {
    Items.find(_query)
        .populate({ path: 'project', select: 'id projectName' })
        .populate({ path: 'endPoint', select: 'id name path' })
        .limit(max_limit) 
        .sort(sortBoj)
        .exec(function (err, items) {
                if (err) {
                    reject(err);
                } else {
                resolve(items);
                }
            })
    });
};
exports.listAllObject = ( query={} ) => {
    var _query={...query,validation:'Object'};
    let sortBy='itemName'
    let sortDirection=-1
    let max_limit = 30000;
    if(query.sortBy){
        sortBy = query.sortBy;
    }
    if(query.sortDirection){
        sortDirection = query.sortDirection;
    }
    var sortBoj={[sortBy]:sortDirection};
    console.log(_query)
    console.log("+++++++++++++++++++++++++++")
    return new Promise((resolve, reject) => {
    Items.find(_query)
        .limit(max_limit) 
        .sort(sortBoj)
        .exec(function (err, items) {
                if (err) {
                    reject(err);
                } else {
                resolve(items);
                }
            })
    });
};
exports.listAllPublish = ( query={} ) => {
    var _query={...query};
    let sortBy='_id'
    let sortDirection=-1
    let max_limit = 30000;
    if(query.sortBy){
        sortBy = query.sortBy;
    }
    if(query.sortDirection){
        sortDirection = query.sortDirection;
    }
    var sortBoj={[sortBy]:sortDirection};
    console.log(_query)
    return new Promise((resolve, reject) => {
    Items.find(_query)
        .select('-createAt -createBy')
        .limit(max_limit) 
        .sort(sortBoj)
        .exec(function (err, items) {
                if (err) {
                    reject(err);
                } else {
                resolve(items);
                }
            })
    });
};
exports.listSuggestions = (query ) => {
    var { search, ...queryWithoutSearch } = query
    var _query=queryWithoutSearch
    if(search){
        _query['$or'] =[ 
            {"itemName": {$regex: search, $options: "i"}},
				{"itemType": {$regex: search, $options: "i"}},
				{"fileTypes": {$in: [search]}},
				{"project": {$regex: search, $options: "i"}},
				{"value": {$regex: search, $options: "i"}}
        ]
    }
    return new Promise((resolve, reject) => {
        Items.find(_query)
            .exec(function (err, items) {
                if (err) {
                    reject(err);
                } else {
                    resolve(items);
                }
            })
    });
};
exports.patchItems = (id, itemsData,extraField={}) => {
    
    var extraQuery =queryFormatter(extraField);
    var queries = {...extraQuery,_id:id}
    return new Promise((resolve, reject) => {
        
        Items.findOne(queries, function (err, items) {
            if (err) reject(err);
            for (let i in itemsData) {
                items[i] = itemsData[i];
            }
            items.save(function (err, updatedItems) {
                if (err) return reject(err);
                resolve(updatedItems);
            });
        });
    })

};

exports.removeById = (itemsId,extraField={}) => {
    var extraQuery =queryFormatter(extraField);
    var queries = {...extraQuery,_id:itemsId}
    return new Promise((resolve, reject) => {
        Items.findOneAndDelete(queries, (err) => {
            if (err) {
                reject(err);
            } else {
                resolve(err);
            }
        });
    });
};



    