const mongoose = require('../../common/services/mongoose.service').mongoose;
const Schema = mongoose.Schema;
const {queryFormatter,queryBuilder_string,
    queryBuilder_number,
    queryBuilder_date,
    queryBuilder_array,
    queryBuilder_range_array} = require("../../common/functions/queryutilMongo")
const logingSchema = new Schema({
    	createBy : { type: String,required:true,default:''},
			createAt : { type: Date,required:true},
			updateBy : { type: String},
			updateAt : { type: Date},
			endPoint : { type: Schema.Types.ObjectId,required:true,ref:'Endpoints'},
			project : { type: Schema.Types.ObjectId,required:true,ref:'Project'},
			status : { type: String,required:true,default:''},
			sentObject : { type: String},
			responseObj : { type: String},
			ip : { type: String}
});

logingSchema.virtual('id').get(function () {
    return this._id.toHexString();
});

// Ensure virtual fields are serialised.
logingSchema.set('toJSON', {
    virtuals: true
});

logingSchema.findById = function (cb) {
    return this.model('Loging').find({id: this.id}, cb);
};

const Loging = mongoose.model('Loging', logingSchema);


exports.findById = (id,extraField) => {
    var extraQuery =queryFormatter(extraField);
    var queries = {...extraQuery,_id:id}
    return Loging.findOne(queries)
        .populate({ path: 'project', select: 'id projectName' })
        .populate({ path: 'endPoint', select: 'id name path' })
        .then((result) => {
            result = result.toJSON();
            delete result._id;
            delete result.__v;
            return result;
        });
};

exports.createLoging = (logingData) => {
    return new Promise((resolve, reject) => {
    
    const loging = new Loging(logingData);
    loging.save(function (err, saved) {
        if (err) {
            return reject(err);
        }
        resolve(saved)
    });
    });
};

exports.list = (perPage, page , query ) => {
        var _query={};
        let sortBy='_id'
        let sortDirection=-1
        
    if(query.createBy){
        let createBy_= queryBuilder_string(query,'createBy');
        _query={..._query,...createBy_}
    }


        if(query.createAt){
            let createAt_= queryBuilder_date(query,'createAt');
            _query={..._query,...createAt_}
        }
        if(query.createAt_array){
            let createAt_a= queryBuilder_range_array(query,'createAt',"date");
            _query={..._query,...createAt_a}
        }
    
    

    if(query.updateBy){
        let updateBy_= queryBuilder_string(query,'updateBy');
        _query={..._query,...updateBy_}
    }


        if(query.updateAt){
            let updateAt_= queryBuilder_date(query,'updateAt');
            _query={..._query,...updateAt_}
        }
        if(query.updateAt_array){
            let updateAt_a= queryBuilder_range_array(query,'updateAt',"date");
            _query={..._query,...updateAt_a}
        }
    
    

    if(query.endPoint){
        let endPoint_= queryBuilder_string(query,'endPoint');
        _query={..._query,...endPoint_}
    }


    if(query.project){
        let project_= queryBuilder_string(query,'project');
        _query={..._query,...project_}
    }


    if(query.status){
        let status_= queryBuilder_string(query,'status');
        _query={..._query,...status_}
    }


    if(query.ip){
        let ip_= queryBuilder_string(query,'ip');
        _query={..._query,...ip_}
    }

        if(query.sortBy){
            sortBy = query.sortBy;
        }
        if(query.sortDirection){
            sortDirection = query.sortDirection;
        }
        var sortBoj={[sortBy]:sortDirection};
        return new Promise((resolve, reject) => {
        Loging.find(_query)
        .populate({ path: 'project', select: 'id projectName' })
        .populate({ path: 'endPoint', select: 'id name path' })
            .limit(perPage)
            .sort(sortBoj)
            .skip(perPage * page)
            .exec(function (err, loging) {
                if (err) {
                    reject(err);
                } else {
                    Loging.countDocuments(_query).exec().then((total)=>{
                        const promises = { docs: loging , count: total ,perpage:perPage,page:page };
                        resolve(promises);
                    }).catch((err2)=>{
                        reject(err2);
                    })
                }
            })
    });
};
exports.listAll = ( query={} ) => {
    var _query={...query};
    let sortBy='_id'
    let sortDirection=-1
    let max_limit = 300;
    if(query.sortBy){
        sortBy = query.sortBy;
    }
    if(query.sortDirection){
        sortDirection = query.sortDirection;
    }
    var sortBoj={[sortBy]:sortDirection};
    return new Promise((resolve, reject) => {
    Loging.find(_query)
        .limit(max_limit) 
        .sort(sortBoj)
        .exec(function (err, loging) {
                if (err) {
                    reject(err);
                } else {
                resolve(loging);
                }
            })
    });
};
exports.listSuggestions = (query ) => {
    var { search, ...queryWithoutSearch } = query
    var _query=queryWithoutSearch
    if(search){
        _query['$or'] =[ 
            {"endPoint": {$regex: search, $options: "i"}},
				{"project": {$regex: search, $options: "i"}},
				{"status": {$regex: search, $options: "i"}},
				{"ip": {$regex: search, $options: "i"}}
        ]
    }
    return new Promise((resolve, reject) => {
        Loging.find(_query)
            .exec(function (err, loging) {
                if (err) {
                    reject(err);
                } else {
                    resolve(loging);
                }
            })
    });
};
exports.patchLoging = (id, logingData,extraField={}) => {
    
    var extraQuery =queryFormatter(extraField);
    var queries = {...extraQuery,_id:id}
    return new Promise((resolve, reject) => {
        
        Loging.findOne(queries, function (err, loging) {
            if (err) reject(err);
            for (let i in logingData) {
                loging[i] = logingData[i];
            }
            loging.save(function (err, updatedLoging) {
                if (err) return reject(err);
                resolve(updatedLoging);
            });
        });
    })

};

exports.removeById = (logingId,extraField={}) => {
    var extraQuery =queryFormatter(extraField);
    var queries = {...extraQuery,_id:logingId}
    return new Promise((resolve, reject) => {
        Loging.findOneAndDelete(queries, (err) => {
            if (err) {
                reject(err);
            } else {
                resolve(err);
            }
        });
    });
};



    