const mongoose = require('../../common/services/mongoose.service').mongoose;
const endPointModel = require('../../src/endpoints/endpoints.model')
const itemsModel = require('../../src/items/items.model')
const fs = require('fs');
var beautify = require("json-beautify");
const Schema = mongoose.Schema;
const {queryFormatter,queryBuilder_string,
    queryBuilder_number,
    queryBuilder_date,
    queryBuilder_array,
    queryBuilder_range_array} = require("../../common/functions/queryutilMongo")
const projectSchema = new Schema({
    	createBy : { type: String,required:true,default:''},
			createAt : { type: Date,required:true},
			updateBy : { type: String},
			updateAt : { type: Date},
			projectName : { type: String,required:true,default:'',maxLength:50,minLength:2},
			details : { type: String},
			basePath : { type: String,required:true,default:''}
});

projectSchema.virtual('id').get(function () {
    return this._id.toHexString();
});

// Ensure virtual fields are serialised.
projectSchema.set('toJSON', {
    virtuals: true
});

projectSchema.findById = function (cb) {
    return this.model('Project').find({id: this.id}, cb);
};

const Project = mongoose.model('Project', projectSchema);


exports.findById = (id,extraField) => {
    var extraQuery =queryFormatter(extraField);
    var queries = {...extraQuery,_id:id}
    return Project.findOne(queries)
        .then((result) => {
            result = result.toJSON();
            delete result._id;
            delete result.__v;
            return result;
        });
};

exports.createProject = (projectData) => {
    return new Promise(async(resolve, reject) => {

    let nameExitit = await this.listAll({projectName:projectData.projectName});
    
    if(nameExitit && nameExitit.length>0){
        return reject("Name exists")
    }

    let basePathExists = await this.listAll({basePath:projectData.basePath});
    
    if(basePathExists && basePathExists.length>0){
        return reject("Base-Path Exists exists")
    }
    
    const project = new Project(projectData);
    project.save(function (err, saved) {
        if (err) {
            return reject(err);
        }
        resolve(saved)
    });
    });
};

exports.list = (perPage, page , query ) => {
        var _query={};
        let sortBy='_id'
        let sortDirection=-1
        
    if(query.createBy){
        let createBy_= queryBuilder_string(query,'createBy');
        _query={..._query,...createBy_}
    }


        if(query.createAt){
            let createAt_= queryBuilder_date(query,'createAt');
            _query={..._query,...createAt_}
        }
        if(query.createAt_array){
            let createAt_a= queryBuilder_range_array(query,'createAt',"date");
            _query={..._query,...createAt_a}
        }
    
    

    if(query.updateBy){
        let updateBy_= queryBuilder_string(query,'updateBy');
        _query={..._query,...updateBy_}
    }


        if(query.updateAt){
            let updateAt_= queryBuilder_date(query,'updateAt');
            _query={..._query,...updateAt_}
        }
        if(query.updateAt_array){
            let updateAt_a= queryBuilder_range_array(query,'updateAt',"date");
            _query={..._query,...updateAt_a}
        }
    
    

    if(query.projectName){
        let projectName_= queryBuilder_string(query,'projectName');
        _query={..._query,...projectName_}
    }


    if(query.basePath){
        let basePath_= queryBuilder_string(query,'basePath');
        _query={..._query,...basePath_}
    }

        if(query.sortBy){
            sortBy = query.sortBy;
        }
        if(query.sortDirection){
            sortDirection = query.sortDirection;
        }
        var sortBoj={[sortBy]:sortDirection};
        return new Promise((resolve, reject) => {
        Project.find(_query)
            .limit(perPage)
            .sort(sortBoj)
            .skip(perPage * page)
            .exec(function (err, project) {
                if (err) {
                    reject(err);
                } else {
                    Project.countDocuments(_query).exec().then((total)=>{
                        const promises = { docs: project , count: total ,perpage:perPage,page:page };
                        resolve(promises);
                    }).catch((err2)=>{
                        reject(err2);
                    })
                }
            })
    });
};
exports.listAll = ( query={} ) => {
    var _query={...query};
    let sortBy='_id'
    let sortDirection=-1
    let max_limit = 300;
    if(query.sortBy){
        sortBy = query.sortBy;
    }
    if(query.sortDirection){
        sortDirection = query.sortDirection;
    }
    var sortBoj={[sortBy]:sortDirection};
    return new Promise((resolve, reject) => {
    Project.find(_query)
        .limit(max_limit) 
        .sort(sortBoj)
        .exec(function (err, project) {
                if (err) {
                    reject(err);
                } else {
                resolve(project);
                }
            })
    });
};
exports.listSuggestions = (query ) => {
    var { search, ...queryWithoutSearch } = query
    var _query=queryWithoutSearch
    if(search){
        _query['$or'] =[ 
            {"projectName": {$regex: search, $options: "i"}},
				{"basePath": {$regex: search, $options: "i"}}
        ]
    }
    return new Promise((resolve, reject) => {
        Project.find(_query)
            .exec(function (err, project) {
                if (err) {
                    reject(err);
                } else {
                    resolve(project);
                }
            })
    });
};
exports.patchProject = (id, projectData,extraField={}) => {
    
    var extraQuery =queryFormatter(extraField);
    var queries = {...extraQuery,_id:id}
    return new Promise((resolve, reject) => {
        
        Project.findOne(queries, function (err, project) {
            if (err) reject(err);
            for (let i in projectData) {
                project[i] = projectData[i];
            }
            project.save(function (err, updatedProject) {
                if (err) return reject(err);
                resolve(updatedProject);
            });
        });
    })

};

exports.publishById = (projectId,extraField={}) => {

    return new Promise(async(resolve, reject) => {
    var endPoints = await endPointModel.listAllPublish({project:projectId});
    var items = await itemsModel.listAllPublish({project:projectId});
    //console.log(endPoints)
    const combinedArray = [];
    endPoints.forEach(obj1 => {
      
        let matchingObj2 = items.filter(obj2 => obj2.endPoint?.toString() === obj1._id.toString());
       // console.log(matchingObj2)
       let tempObj={}
      
        if (matchingObj2.length>0) {
         
            tempObj =  { ...obj1.toObject(), exItems: matchingObj2 }
         
            
        } else {
            tempObj =  { ...obj1.toObject(), exItems: [] }
        }
  
        combinedArray.push(tempObj);

    });

const filePath = './download/routes.json'; // Path to the file

try {
    fs.writeFileSync(filePath,beautify(combinedArray, null, 2, 100));
    console.log('File saved successfully.');
    resolve(combinedArray);
} catch (err) {
    console.error('Error saving file:', err);
    reject(err)
}
          //  }
       // });
    });
};



    